use serde::ser::{Serialize, SerializeMap, Serializer};
use serde_json::{Map, Value};
use std::collections::HashMap;
use zvariant::OwnedValue;

/// Serialize actions
/// # Errors
/// Will return an empty map if there are no actions
pub fn serialize_actions<S>(actions: &[String], serializer: S) -> Result<S::Ok, S::Error>
where
    S: Serializer,
{
    let mut map = Map::new();

    // Actions are in pairs: [id, label, id, label, ...]
    for pair in actions.chunks(2) {
        if let [id, label] = pair {
            map.insert(id.clone(), Value::String(label.clone()));
        }
    }

    map.serialize(serializer)
}

/// Serialize hints
/// # Errors
/// Will return an empty map if there are no hints
pub fn serialize_hints<S>(
    hints: &HashMap<String, OwnedValue>,
    serializer: S,
) -> Result<S::Ok, S::Error>
where
    S: Serializer,
{
    // Start serializing the map
    let mut map = serializer.serialize_map(Some(hints.len()))?;
    for (key, value) in hints {
        // Serialize each entry as desired
        map.serialize_entry(key, &HintValueSerializer(value))?;
    }
    map.end()
}

// A custom struct to handle serialization of OwnedValue
struct HintValueSerializer<'a>(&'a OwnedValue);

impl Serialize for HintValueSerializer<'_> {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        let signature = self.0.value_signature().to_string(); // Get signature

        // Extract the raw value correctly
        let raw_value = if let Ok(v) = self.0.downcast_ref::<u8>() {
            Value::from(v) // BYTE: y
        } else if let Ok(v) = self.0.downcast_ref::<bool>() {
            Value::Bool(v) // BOOLEAN: b
        } else if let Ok(v) = self.0.downcast_ref::<i16>() {
            Value::from(v) // INT16: n
        } else if let Ok(v) = self.0.downcast_ref::<u16>() {
            Value::from(v) // UINT16: q
        } else if let Ok(v) = self.0.downcast_ref::<i32>() {
            Value::from(v) // INT32: i
        } else if let Ok(v) = self.0.downcast_ref::<u32>() {
            Value::from(v) // UINT32: u
        } else if let Ok(v) = self.0.downcast_ref::<i64>() {
            Value::from(v) // INT64: x
        } else if let Ok(v) = self.0.downcast_ref::<u64>() {
            Value::from(v) // UINT64: t
        } else if let Ok(v) = self.0.downcast_ref::<f64>() {
            Value::from(v) // DOUBLE: d
        } else if let Ok(v) = self.0.downcast_ref::<String>() {
            Value::String(v.clone()) // STRING: s
        } else if let Ok(v) = self.0.downcast_ref::<&str>() {
            Value::String(v.to_string()) // str
        } else {
            Value::Null // Unsupported types: fallback to Null
        }; // Not implemented: UNIX_FD: h, OBJECT_PATH: o, SIGNATURE: g

        // Serialize the final structure as a map
        let mut map = serializer.serialize_map(Some(2))?;
        map.serialize_entry("signature", &signature)?;
        map.serialize_entry("value", &raw_value)?;
        map.end()
    }
}
